@@ -1,15 +1,13 @@
-import logging
 import typing as t
-from time import perf_counter
-from typing import List
 
+import bottleneck as bn
 import h5py
 import numpy as np
 import pandas as pd
-from agora.abc import ParametersABC, ProcessABC
+
+from agora.abc import ParametersABC, StepABC
 from agora.io.cells import Cells
 from agora.io.writer import Writer, load_attributes
-
 from aliby.tile.tiler import Tiler
 from extraction.core.functions.defaults import exparams_from_meta
 from extraction.core.functions.distributors import reduce_z, trap_apply
@@ -19,7 +17,7 @@ from extraction.core.functions.loaders import (
     load_redfuns,
 )
 
-# Define types
+# define types
 reduction_method = t.Union[t.Callable, str, None]
 extraction_tree = t.Dict[
     str, t.Dict[reduction_method, t.Dict[str, t.Collection]]
@@ -28,7 +26,7 @@ extraction_result = t.Dict[
     str, t.Dict[reduction_method, t.Dict[str, t.Dict[str, pd.Series]]]
 ]
 
-# Global parameters used to load functions that either analyse cells or their background. These global parameters both allow the functions to be stored in a dictionary for access only on demand and to be defined simply in extraction/core/functions.
+# Global variables used to load functions that either analyse cells or their background. These global variables both allow the functions to be stored in a dictionary for access only on demand and to be defined simply in extraction/core/functions.
 CELL_FUNS, TRAPFUNS, FUNS = load_funs()
 CUSTOM_FUNS, CUSTOM_ARGS = load_custom_args()
 RED_FUNS = load_redfuns()
@@ -38,9 +36,7 @@ RED_FUNS = load_redfuns()
 
 
 class ExtractorParameters(ParametersABC):
-    """
-    Base class to define parameters for extraction
-    """
+    """Base class to define parameters for extraction."""
 
     def __init__(
         self,
@@ -49,13 +45,15 @@ class ExtractorParameters(ParametersABC):
         multichannel_ops: t.Dict = {},
     ):
         """
+        Initialise.
+
         Parameters
         ----------
         tree: dict
             Nested dictionary indicating channels, reduction functions and
             metrics to be used.
-            str channel -> U(function,None) reduction -> str metric
-            If not of depth three, tree will be filled with Nones.
+            str channel -> U(function, None) reduction -> str metric
+            If not of depth three, tree will be filled with None.
         sub_bg: set
         multichannel_ops: dict
         """
@@ -66,7 +64,7 @@ class ExtractorParameters(ParametersABC):
     @staticmethod
     def guess_from_meta(store_name: str, suffix="fast"):
         """
-        Find the microscope used from the h5 metadata
+        Find the microscope name from the h5 metadata.
 
         Parameters
         ----------
@@ -89,27 +87,20 @@ class ExtractorParameters(ParametersABC):
         return cls(**exparams_from_meta(meta))
 
 
-class Extractor(ProcessABC):
+class Extractor(StepABC):
     """
-    The Extractor applies a metric, such as area or median, to cells identified in the image tiles using the cell masks.
+    Apply a metric to cells identified in the tiles.
 
-    Its methods therefore require both tile images and masks.
+    Using the cell masks, the Extractor applies a metric, such as area or median, to cells identified in the image tiles.
 
-    Usually one metric is applied to the masked area in a tile, but there are metrics that depend on the whole tile.
+    Its methods require both tile images and masks.
 
-    Extraction follows a three-level tree structure. Channels, such as GFP, are the root level; the second level is the reduction algorithm, such as maximum projection; the last level is the metric - the specific operation to apply to the cells in the image identified by the mask, such as median, which is the median value of the pixels in each cell.
+    Usually the metric is applied to only a tile's masked area, but some metrics depend on the whole tile.
 
-    Parameters
-    ----------
-    parameters: core.extractor Parameters
-        Parameters that include with channels, reduction and
-        extraction functions to use.
-    store: str
-        Path to hdf5 storage file. Must contain cell outlines.
-    tiler: pipeline-core.core.segmentation tiler
-        Class that contains or fetches the image to be used for segmentation.
+    Extraction follows a three-level tree structure. Channels, such as GFP, are the root level; the reduction algorithm, such as maximum projection, is the second level; the specific metric, or operation, to apply to the masks, such as mean, is the third level.
     """
 
+    # TODO Alan: Move this to a location with the SwainLab defaults
     default_meta = {
         "pixel_size": 0.236,
         "z_size": 0.6,
@@ -127,10 +118,13 @@ class Extractor(ProcessABC):
 
         Parameters
         ----------
-        parameters: ExtractorParameters object
+        parameters: core.extractor Parameters
+            Parameters that include the channels, reduction and
+            extraction functions.
         store: str
-            Name of h5 file
-        tiler: Tiler object
+            Path to the h5 file containing the cell masks.
+        tiler: pipeline-core.core.segmentation tiler
+            Class that contains or fetches the images used for segmentation.
         """
         self.params = parameters
         if store:
@@ -150,7 +144,7 @@ class Extractor(ProcessABC):
         store: str,
         tiler: Tiler,
     ):
-        # initate from tiler
+        """Initiate from a tiler instance."""
         return cls(parameters, store=store, tiler=tiler)
 
     @classmethod
@@ -160,12 +154,12 @@ class Extractor(ProcessABC):
         store: str,
         img_meta: tuple,
     ):
-        # initiate from image
+        """Initiate from images."""
         return cls(parameters, store=store, tiler=Tiler(*img_meta))
 
     @property
     def channels(self):
-        # returns a tuple of strings of the available channels
+        """Get a tuple of the available channels."""
         if not hasattr(self, "_channels"):
             if type(self.params.tree) is dict:
                 self._channels = tuple(self.params.tree.keys())
@@ -177,14 +171,18 @@ class Extractor(ProcessABC):
 
     @property
     def group(self):
-        # returns path within h5 file
+        """Return path within the h5 file."""
         if not hasattr(self, "_out_path"):
             self._group = "/extraction/"
         return self._group
 
     def load_custom_funs(self):
         """
-        Define any custom functions to be functions of cell_masks and trap_image only.
+        Incorporate the extra arguments of custom functions into their definitions.
+
+        Normal functions only have cell_masks and trap_image as their
+        arguments, and here custom functions are made the same by
+        setting the values of their extra arguments.
 
         Any other parameters are taken from the experiment's metadata and automatically applied. These parameters therefore must be loaded within an Extractor instance.
         """
@@ -204,12 +202,13 @@ class Extractor(ProcessABC):
             k: {k2: self.get_meta(k2) for k2 in v}
             for k, v in CUSTOM_ARGS.items()
         }
-        # define custom functions - those with extra arguments other than cell_masks and trap_image - as functions of two variables
+        # define custom functions
         self._custom_funs = {}
         for k, f in CUSTOM_FUNS.items():
 
             def tmp(f):
                 # pass extra arguments to custom function
+                # return a function of cell_masks and trap_image
                 return lambda cell_masks, trap_image: trap_apply(
                     f,
                     cell_masks,
@@ -220,13 +219,14 @@ class Extractor(ProcessABC):
             self._custom_funs[k] = tmp(f)
 
     def load_funs(self):
+        """Define all functions, including custum ones."""
         self.load_custom_funs()
         self._all_cell_funs = set(self._custom_funs.keys()).union(CELL_FUNS)
         # merge the two dicts
         self._all_funs = {**self._custom_funs, **FUNS}
 
     def load_meta(self):
-        # load metadata from h5 file whose name is given by self.local
+        """Load metadata from h5 file."""
         self.meta = load_attributes(self.local)
 
     def get_tiles(
@@ -237,19 +237,18 @@ class Extractor(ProcessABC):
         **kwargs,
     ) -> t.Optional[np.ndarray]:
         """
-        Finds traps for a given time point and given channels and z-stacks.
-        Returns None if no traps are found.
+        Find tiles for a given time point, channels, and z-stacks.
 
         Any additional keyword arguments are passed to tiler.get_tiles_timepoint
 
         Parameters
         ----------
         tp: int
-            Time point of interest
+            Time point of interest.
         channels: list of strings (optional)
-            Channels of interest
+            Channels of interest.
         z: list of integers (optional)
-            Indices for the z-stacks of interest
+            Indices for the z-stacks of interest.
         """
         if channels is None:
             # find channels from tiler
@@ -262,16 +261,16 @@ class Extractor(ProcessABC):
             channel_ids = None
         if z is None:
             # gets the tiles data via tiler
-            z: t.List[int] = list(range(self.tiler.shape[-1]))
-        tiles = (
+            z = list(range(self.tiler.shape[-3]))
+        res = (
             self.tiler.get_tiles_timepoint(
                 tp, channels=channel_ids, z=z, **kwargs
             )
             if channel_ids
             else None
         )
-        # data arranged as (traps, channels, timepoints, X, Y, Z)
-        return tiles
+        # data arranged as (tiles, channels, time points, X, Y, Z)
+        return res
 
     def extract_traps(
         self,
@@ -286,9 +285,9 @@ class Extractor(ProcessABC):
         Parameters
         ----------
         traps: list of arrays
-            List of images.
+            t.List of images.
         masks: list of arrays
-            List of masks.
+            t.List of masks.
         metric: str
             Metric to extract.
         labels: dict
@@ -299,11 +298,10 @@ class Extractor(ProcessABC):
         Returns
         -------
         res_idx: a tuple of tuples
-            A two-tuple of a tuple of results and a tuple with the corresponding trap_id and cell labels
+            A two-tuple comprising a tuple of results and a tuple of the tile_id and cell labels
         """
         if labels is None:
-            print("Warning: No labels given. Sorting cells using index.")
-
+            self._log("No labels given. Sorting cells using index.")
         cell_fun = True if metric in self._all_cell_funs else False
         idx = []
         results = []
@@ -328,13 +326,15 @@ class Extractor(ProcessABC):
 
     def extract_funs(
         self,
-        traps: List[np.array],
-        masks: List[np.array],
+        traps: t.List[np.array],
+        masks: t.List[np.array],
         metrics: t.List[str],
         **kwargs,
     ) -> t.Dict[str, pd.Series]:
         """
-        Returns dict with metrics as key and metrics applied to data as values for data from one timepoint.
+        Return dict with metrics as key and metrics applied to data as values.
+
+        Data from one time point is used.
         """
         d = {
             metric: self.extract_traps(
@@ -356,8 +356,8 @@ class Extractor(ProcessABC):
 
         Parameters
         ----------
-        traps: array
-            An array of image data arranged as (traps, X, Y, Z)
+        tiles_data: array
+            An array of image data arranged as (tiles, X, Y, Z)
         masks: list of arrays
             An array of masks for each trap: one per cell at the trap
         red_metrics: dict
@@ -368,21 +368,20 @@ class Extractor(ProcessABC):
 
         Returns
         ------
-        Dictionary of dataframes with the corresponding reductions and metrics nested.
+        Dict of dataframes with the corresponding reductions and metrics nested.
         """
         # create dict with keys naming the reduction in the z-direction and the reduced data as values
-        reduced_traps = {}
+        reduced_tiles_data = {}
         if traps is not None:
             for red_fun in red_metrics.keys():
-                reduced_traps[red_fun] = [
-                    self.reduce_dims(trap, method=RED_FUNS[red_fun])
-                    for trap in traps
+                reduced_tiles_data[red_fun] = [
+                    self.reduce_dims(tile_data, method=RED_FUNS[red_fun])
+                    for tile_data in traps
                 ]
-
         d = {
             red_fun: self.extract_funs(
                 metrics=metrics,
-                traps=reduced_traps.get(red_fun, [None for _ in masks]),
+                traps=reduced_tiles_data.get(red_fun, [None for _ in masks]),
                 masks=masks,
                 **kwargs,
             )
@@ -395,18 +394,21 @@ class Extractor(ProcessABC):
     ) -> np.ndarray:
         """
         Collapse a z-stack into 2d array using method.
+
         If method is None, return the original data.
 
         Parameters
         ----------
         img: array
-            An array of the image data arranged as (X, Y, Z)
+            An array of the image data arranged as (X, Y, Z).
         method: function
-            The reduction function
+            The reduction function.
         """
         reduced = img
         if method is not None:
             reduced = reduce_z(img, method)
+        if reduced.shape[0] < 10:
+            print("ahoy")
         return reduced
 
     def extract_tp(
@@ -419,7 +421,7 @@ class Extractor(ProcessABC):
         **kwargs,
     ) -> t.Dict[str, t.Dict[str, t.Dict[str, tuple]]]:
         """
-        Core extraction method for an individual time-point.
+        Extract for an individual time point.
 
         Parameters
         ----------
@@ -449,8 +451,8 @@ class Extractor(ProcessABC):
             The first tuple is the result of applying the metrics to a particular cell or trap; the second tuple is either (trap_id, cell_label) for a metric applied to a cell or a trap_id for a metric applied to a trap.
 
             An example is d["GFP"]["np_max"]["mean"][0], which gives a tuple of the calculated mean GFP fluorescence for all cells.
-
         """
+        # TODO Can we split the different extraction types into sub-methods to make this easier to read?
         if tree is None:
             # use default
             tree: extraction_tree = self.params.tree
@@ -460,7 +462,6 @@ class Extractor(ProcessABC):
         tree_chs = (*ch_tree,)
         # create a Cells object to extract information from the h5 file
         cells = Cells(self.local)
-
         # find the cell labels and store as dict with trap_ids as keys
         if labels is None:
             raw_labels = cells.labels_at_time(tp)
@@ -468,31 +469,37 @@ class Extractor(ProcessABC):
                 trap_id: raw_labels.get(trap_id, [])
                 for trap_id in range(cells.ntraps)
             }
-
         # find the cell masks for a given trap as a dict with trap_ids as keys
         if masks is None:
             raw_masks = cells.at_time(tp, kind="mask")
             masks = {trap_id: [] for trap_id in range(cells.ntraps)}
             for trap_id, cells in raw_masks.items():
                 if len(cells):
-                    masks[trap_id] = np.dstack(np.array(cells)).astype(bool)
+                    masks[trap_id] = np.stack(np.array(cells)).astype(bool)
         # convert to a list of masks
         masks = [np.array(v) for v in masks.values()]
-
         # find image data at the time point
-        # stored as an array arranged as (traps, channels, timepoints, X, Y, Z)
+        # stored as an array arranged as (traps, channels, time points, X, Y, Z)
         tiles = self.get_tiles(tp, tile_shape=tile_size, channels=tree_chs)
-
         # generate boolean masks for background as a list with one mask per trap
-        bgs = []
+        bgs = np.array([])
         if self.params.sub_bg:
-            bgs = [
-                ~np.sum(m, axis=2).astype(bool)
-                if np.any(m)
-                else np.zeros((tile_size, tile_size))
-                for m in masks
-            ]
-
+            # bgs = [
+            #     ~np.sum(m, axis=0).astype(bool)
+            #     if np.any(m)
+            #     else np.zeros((tile_size, tile_size)).astype(bool)
+            #     for m in masks
+            # ]
+            bgs = ~np.array(
+                list(
+                    map(
+                        lambda x: np.sum(x, axis=0)
+                        if np.any(x)
+                        else np.zeros((tile_size, tile_size)),
+                        masks,
+                    )
+                )
+            ).astype(bool)
         # perform extraction by applying metrics
         d = {}
         self.img_bgsub = {}
@@ -500,31 +507,38 @@ class Extractor(ProcessABC):
             # NB ch != is necessary for threading
             if ch != "general" and tiles is not None and len(tiles):
                 # image data for all traps and z sections for a particular channel
-                # as an array arranged as (no traps, X, Y, no Z channels)
+                # as an array arranged as (tiles, Z, X, Y, )
                 img = tiles[:, tree_chs.index(ch), 0]
             else:
                 img = None
             # apply metrics to image data
             d[ch] = self.reduce_extract(
-                red_metrics=red_metrics,
                 traps=img,
                 masks=masks,
+                red_metrics=red_metrics,
                 labels=labels,
                 **kwargs,
             )
             # apply metrics to image data with the background subtracted
-            if bgs and ch in self.params.sub_bg and img is not None:
+            if bgs.any() and ch in self.params.sub_bg and img is not None:
                 # calculate metrics with subtracted bg
                 ch_bs = ch + "_bgsub"
-                self.img_bgsub[ch_bs] = []
-                for trap, bg in zip(img, bgs):
-                    bg_fluo = np.zeros_like(trap)
-                    not_cell = np.where(bg)
-                    # skip for empty traps
-                    if len(not_cell[0]):
-                        bg_fluo = np.median(trap[not_cell], axis=0)
-                    # subtract median background
-                    self.img_bgsub[ch_bs].append(trap - bg_fluo)
+                # subtract median background
+
+                self.img_bgsub[ch_bs] = np.moveaxis(
+                    np.stack(
+                        list(
+                            map(
+                                lambda tile, mask: np.moveaxis(tile, 0, -1)
+                                - bn.median(tile[:, mask], axis=1),
+                                img,
+                                bgs,
+                            )
+                        )
+                    ),
+                    -1,
+                    1,
+                )  # End with tiles, z, y, x
                 # apply metrics to background-corrected data
                 d[ch_bs] = self.reduce_extract(
                     red_metrics=ch_tree[ch],
@@ -533,8 +547,7 @@ class Extractor(ProcessABC):
                     labels=labels,
                     **kwargs,
                 )
-
-        # apply any metrics that use multiple channels (eg pH calculations)
+        # apply any metrics using multiple channels, such as pH calculations
         for name, (
             chs,
             merge_fun,
@@ -556,56 +569,51 @@ class Extractor(ProcessABC):
                     labels=labels,
                     **kwargs,
                 )
-
         return d
 
-    def get_imgs(self, channel: t.Optional[str], traps, channels=None):
+    def get_imgs(self, channel: t.Optional[str], tiles, channels=None):
         """
-        Returns the image from a correct source, either raw or bgsub
+        Return image from a correct source, either raw or bgsub.
 
         Parameters
         ----------
         channel: str
             Name of channel to get.
-        traps: ndarray
-            An array of the image data having dimensions of (trap_id, channel, tp, tile_size, tile_size, n_zstacks).
+        tiles: ndarray
+            An array of the image data having dimensions of (tile_id, channel, tp, tile_size, tile_size, n_zstacks).
         channels: list of str (optional)
-            List of available channels.
+            t.List of available channels.
 
         Returns
         -------
         img: ndarray
-            An array of image data with dimensions (no traps, X, Y, no Z channels)
+            An array of image data with dimensions (no tiles, X, Y, no Z channels)
         """
         if channels is None:
             channels = (*self.params.tree,)
-        if channel in channels:
-            return traps[:, channels.index(channel), 0]
+        if channel in channels:  # TODO start here to fetch channel using regex
+            return tiles[:, channels.index(channel), 0]
         elif channel in self.img_bgsub:
             return self.img_bgsub[channel]
 
-    def run_tp(self, tp, **kwargs):
-        """
-        Wrapper to add compatiblibility with other steps of the pipeline.
-        """
-        return self.run(tps=[tp], **kwargs)
-
-    def run(
+    def _run_tp(
         self,
+        tps: t.List[int] = None,
         tree=None,
-        tps: List[int] = None,
         save=True,
         **kwargs,
     ) -> dict:
         """
+        Wrapper to add compatibility with other steps of the pipeline.
+
         Parameters
         ----------
-        tree: dict
+        tps: list of int (optional)
+            Time points to include.
+        tree: dict (optional)
             Nested dictionary indicating channels, reduction functions and
             metrics to be used.
             For example: {'general': {'None': ['area', 'volume', 'eccentricity']}}
-        tps: list of int (optional)
-            Time points to include.
         save: boolean (optional)
             If True, save results to h5 file.
         kwargs: keyword arguments (optional)
@@ -620,6 +628,8 @@ class Extractor(ProcessABC):
             tree = self.params.tree
         if tps is None:
             tps = list(range(self.meta["time_settings/ntimepoints"][0]))
+        elif isinstance(tps, int):
+            tps = [tps]
         # store results in dict
         d = {}
         for tp in tps:
@@ -666,7 +676,7 @@ class Extractor(ProcessABC):
         self.writer.id_cache.clear()
 
     def get_meta(self, flds: t.Union[str, t.Collection]):
-        # Obtain metadata for one or multiple fields
+        """Obtain metadata for one or multiple fields."""
         if isinstance(flds, str):
             flds = [flds]
         meta_short = {k.split("/")[-1]: v for k, v in self.meta.items()}
@@ -680,7 +690,7 @@ def flatten_nesteddict(
     nest: dict, to="series", tp: int = None
 ) -> t.Dict[str, pd.Series]:
     """
-    Converts a nested extraction dict into a dict of pd.Series
+    Convert a nested extraction dict into a dict of pd.Series.
 
     Parameters
     ----------
@@ -689,7 +699,7 @@ def flatten_nesteddict(
     to: str (optional)
         Specifies the format of the output, either pd.Series (default) or a list
     tp: int
-        Timepoint used to name the pd.Series
+        Time point used to name the pd.Series
 
     Returns
     -------
@@ -709,6 +719,7 @@ def flatten_nesteddict(
 class hollowExtractor(Extractor):
     """
     Extractor that only cares about receiving images and masks.
+
     Used for testing.
     """
 
