@@ -4,18 +4,16 @@ from typing import Dict, List, Union
 import h5py
 import numpy as np
 import pandas as pd
+from tqdm import tqdm
+
 from agora.abc import ParametersABC, ProcessABC
 from agora.io.cells import Cells
 from agora.io.signal import Signal
 from agora.io.writer import Writer
-from tqdm import tqdm
-
 from postprocessor.core.abc import get_parameters, get_process
-from postprocessor.core.lineageprocess import (
-    LineageProcessParameters,
-)
-from postprocessor.core.reshapers.merger import merger, mergerParameters
-from postprocessor.core.reshapers.picker import picker, pickerParameters
+from postprocessor.core.lineageprocess import LineageProcessParameters
+from postprocessor.core.reshapers.merger import Merger, MergerParameters
+from postprocessor.core.reshapers.picker import Picker, PickerParameters
 
 
 class PostProcessorParameters(ParametersABC):
@@ -44,6 +42,7 @@ class PostProcessorParameters(ParametersABC):
 
     @classmethod
     def default(cls, kind=[]):
+        """Sequential postprocesses to be operated"""
         targets = {
             "prepost": {
                 "merger": "/extraction/general/None/area",
@@ -56,26 +55,16 @@ class PostProcessorParameters(ParametersABC):
                         "/extraction/general/None/volume",
                     ],
                 ],
-                # [
-                #     "savgol",
-                #     [
-                #         "/extraction/general/None/volume",
-                #     ],
-                # ],
                 [
                     "dsignal",
                     [
                         "/extraction/general/None/volume",
-                        # "/postprocessing/savgol/extraction_general_None_volume",
                     ],
                 ],
                 [
                     "bud_metric",
                     [
                         "/extraction/general/None/volume",
-                        # "/postprocessing/dsignal/postprocessing_savgol_extraction_general_None_volume",
-                        # "/postprocessing/savgol/extraction_general_None_volume",
-                        # "/postprocessing/dsignal/extraction_general_None_volume",
                     ],
                 ],
                 [
@@ -84,43 +73,24 @@ class PostProcessorParameters(ParametersABC):
                         "/postprocessing/bud_metric/extraction_general_None_volume",
                     ],
                 ],
-                # [
-                #     "aggregate",
-                #     [
-                #         [
-                #             "/extraction/general/None/volume",
-                #             "postprocessing/dsignal/extraction_general_None_volume",
-                #         ],
-                #     ],
-                # ],
             ],
         }
         param_sets = {
             "prepost": {
-                "merger": mergerParameters.default(),
-                "picker": pickerParameters.default(),
+                "merger": MergerParameters.default(),
+                "picker": PickerParameters.default(),
             }
         }
         outpaths = {}
         outpaths["aggregate"] = "/postprocessing/experiment_wide/aggregated/"
 
         if "ph_batman" in kind:
-            # targets["processes"]["bud_metric"].append(
-            #     [
-            #         [
-            #             "/extraction/em_ratio/np_max/mean",
-            #             "/extraction/em_ratio/np_max/median",
-            #         ],
-            #     ]
-            # )
             targets["processes"]["dsignal"].append(
                 [
                     "/extraction/em_ratio/np_max/mean",
                     "/extraction/em_ratio/np_max/median",
                     "/extraction/em_ratio_bgsub/np_max/mean",
                     "/extraction/em_ratio_bgsub/np_max/median",
-                    # "/postprocessing/bud_metric/extraction_em_ratio_np_max_mean",
-                    # "/postprocessing/bud_metric/extraction_em_ratio_np_max_median",
                 ]
             )
             targets["processes"]["aggregate"].append(
@@ -132,10 +102,6 @@ class PostProcessorParameters(ParametersABC):
                         "/extraction/em_ratio_bgsub/np_max/median",
                         "/extraction/gsum/np_max/median",
                         "/extraction/gsum/np_max/mean",
-                        # "postprocessing/bud_metric/extraction_em_ratio_np_max_mean",
-                        # "postprocessing/bud_metric/extraction_em_ratio_np_max_median",
-                        # "postprocessing/dsignal/postprocessing_bud_metric_extraction_em_ratio_np_max_median",
-                        # "postprocessing/dsignal/postprocessing_bud_metric_extraction_em_ratio_np_max_mean",
                     ]
                 ],
             )
@@ -159,12 +125,12 @@ class PostProcessor(ProcessABC):
             if not isinstance(dicted_params[k], dict):
                 dicted_params[k] = dicted_params[k].to_dict()
 
-        self.merger = merger(
-            mergerParameters.from_dict(dicted_params["merger"])
+        self.merger = Merger(
+            MergerParameters.from_dict(dicted_params["merger"])
         )
 
-        self.picker = picker(
-            pickerParameters.from_dict(dicted_params["picker"]),
+        self.picker = Picker(
+            PickerParameters.from_dict(dicted_params["picker"]),
             cells=Cells.from_source(filename),
         )
         self.classfun = {
@@ -178,6 +144,7 @@ class PostProcessor(ProcessABC):
         self.targets = parameters["targets"]
 
     def run_prepost(self):
+        # TODO Split function
         """Important processes run before normal post-processing ones"""
 
         merge_events = self.merger.run(
@@ -191,9 +158,8 @@ class PostProcessor(ProcessABC):
         ]
         self._writer.write("modifiers/merges", data=changes_history)
 
-        with h5py.File(
-            self._filename, "a"
-        ) as f:  # TODO Remove this once done tweaking
+        # TODO Remove this once test is wriiten for consecutive postprocesses
+        with h5py.File(self._filename, "a") as f:
             if "modifiers/picks" in f:
                 del f["modifiers/picks"]
 
@@ -201,16 +167,20 @@ class PostProcessor(ProcessABC):
             self._signal[self.targets["prepost"]["picker"][0]]
         )
 
-        mothers, daughters = np.array(self.picker.mothers), np.array(
-            self.picker.daughters
-        )
+        combined_idx = ([], [], [])
+        trap, mother, daughter = combined_idx
+
+        lineage = self.picker.cells.mothers_daughters
+
+        if lineage.any():
+            trap, mother, daughter = lineage.T
+            combined_idx = np.vstack((trap, mother, daughter))
+
+        trap_mother = np.vstack((trap, mother)).T
+        trap_daughter = np.vstack((trap, daughter)).T
 
         multii = pd.MultiIndex.from_arrays(
-            (
-                np.append(mothers, daughters[:, 1].reshape(-1, 1), axis=1).T
-                if daughters.any()
-                else [[], [], []]
-            ),
+            combined_idx,
             names=["trap", "mother_label", "daughter_label"],
         )
         self._writer.write(
@@ -219,10 +189,11 @@ class PostProcessor(ProcessABC):
             overwrite="overwrite",
         )
 
-        # apply merge to mother-daughter
-        moset = set([tuple(x) for x in mothers])
-        daset = set([tuple(x) for x in daughters])
+        # apply merge to mother-trap_daughter
+        moset = set([tuple(x) for x in trap_mother])
+        daset = set([tuple(x) for x in trap_daughter])
         picked_set = set([tuple(x) for x in indices])
+
         with h5py.File(self._filename, "a") as f:
             merge_events = f["modifiers/merges"][()]
         multii = pd.MultiIndex(
@@ -231,6 +202,7 @@ class PostProcessor(ProcessABC):
             names=["trap", "mother_label", "daughter_label"],
         )
         self.lineage_merged = multii
+
         if merge_events.any():
 
             def search(a, b):
@@ -245,19 +217,25 @@ class PostProcessor(ProcessABC):
                 if (
                     tuple(source) in moset
                 ):  # update mother to lowest positive index among the two
-                    mother_ids = search(mothers, source)
-                    mothers[mother_ids] = (
+                    mother_ids = search(trap_mother, source)
+                    trap_mother[mother_ids] = (
                         target[0],
-                        self.pick_mother(mothers[mother_ids][0][1], target[1]),
+                        self.pick_mother(
+                            trap_mother[mother_ids][0][1], target[1]
+                        ),
                     )
                 if tuple(source) in daset:
-                    daughters[search(daughters, source)] = target
+                    trap_daughter[search(trap_daughter, source)] = target
                 if tuple(source) in picked_set:
                     indices[search(indices, source)] = target
 
             self.lineage_merged = pd.MultiIndex.from_arrays(
                 np.unique(
-                    np.append(mothers, daughters[:, 1].reshape(-1, 1), axis=1),
+                    np.append(
+                        trap_mother,
+                        trap_daughter[:, 1].reshape(-1, 1),
+                        axis=1,
+                    ),
                     axis=0,
                 ).T,
                 names=["trap", "mother_label", "daughter_label"],
@@ -271,7 +249,7 @@ class PostProcessor(ProcessABC):
         self._writer.write(
             "modifiers/picks",
             data=pd.MultiIndex.from_arrays(
-                # FIXME there seem to be repeated indices, it should not be the case
+                # TODO Check if multiindices are still repeated
                 np.unique(indices, axis=0).T if indices.any() else [[], []],
                 names=["trap", "cell_label"],
             ),
@@ -290,15 +268,10 @@ class PostProcessor(ProcessABC):
         return x
 
     def run(self):
-        # import cProfile
-        # import pstats
-
-        # profile = cProfile.Profile()
-        # profile.enable()
-
+        # TODO Documentation :) + Split
         self.run_prepost()
 
-        for process, datasets in tqdm(enumerate(self.targets["processes"])):
+        for process, datasets in tqdm(self.targets["processes"]):
             if process in self.parameters["param_sets"].get(
                 "processes", {}
             ):  # If we assigned parameters
@@ -314,7 +287,8 @@ class PostProcessor(ProcessABC):
                     # self.parameters.lineage_location
                 )
                 loaded_process = self.classfun[process](parameters)
-                loaded_process.load_lineage(lineage)
+                loaded_process.lineage = lineage
+
             else:
                 loaded_process = self.classfun[process](parameters)
 
@@ -383,9 +357,3 @@ class PostProcessor(ProcessABC):
         metadata: Dict,
     ):
         self._writer.write(path, result, meta=metadata, overwrite="overwrite")
-
-
-def _if_dict(item):
-    if hasattr(item, "to_dict"):
-        item = item.to_dict()
-    return item
