@@ -4,7 +4,7 @@ from postprocessor.core.abc import PostProcessABC
 from postprocessor.core.functions.tracks import get_joinable
 
 
-class mergerParameters(ParametersABC):
+class MergerParameters(ParametersABC):
     """
     :param tol: float or int threshold of average (prediction error/std) necessary
         to consider two tracks the same. If float is fraction of first track,
@@ -22,7 +22,7 @@ class mergerParameters(ParametersABC):
     }
 
 
-class merger(PostProcessABC):
+class Merger(PostProcessABC):
     """
     Combines rows of tracklet that are likely to be the same.
     """
@@ -34,8 +34,4 @@ class merger(PostProcessABC):
         joinable = []
         if signal.shape[1] > 4:
             joinable = get_joinable(signal, tol=self.parameters.tolerance)
-        # merged, _ = merge_tracks(signal)  # , min_len=self.window + 1)
-        # indices = (*zip(*merged.index.tolist()),)
-        # names = merged.index.names
-        # return {name: ids for name, ids in zip(names, indices)}
         return joinable
