@@ -13,62 +13,64 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import seaborn as sns
-from postprocessor.chainer import Chainer
 from pathos.multiprocessing import Pool
 
+from postprocessor.chainer import Chainer
+
 
 class Grouper(ABC):
     """Base grouper class."""
 
-    files = []
-
     def __init__(self, dir: Union[str, PosixPath]):
+        """Find h5 files and load a chain for each one."""
         path = Path(dir)
+        assert path.exists(), f"{str(dir)} does not exist"
         self.name = path.name
-        assert path.exists(), "Dir does not exist"
         self.files = list(path.glob("*.h5"))
         assert len(self.files), "No valid h5 files in dir"
         self.load_chains()
 
     def load_chains(self) -> None:
-        # Sets self.chainers
+        """Load a chain for each position, or h5 file."""
         self.chainers = {f.name[:-3]: Chainer(f) for f in self.files}
 
     @property
     def fsignal(self) -> Chainer:
-        # Returns first signal
+        """Get first chain."""
         return list(self.chainers.values())[0]
 
     @property
     def ntimepoints(self) -> int:
+        """Find number of time points."""
         return max([s.ntimepoints for s in self.chainers.values()])
 
     @property
     def tintervals(self) -> float:
+        """Find the maximum time interval for all chains."""
         tintervals = set([s.tinterval / 60 for s in self.chainers.values()])
         assert (
             len(tintervals) == 1
         ), "Not all chains have the same time interval"
-
         return max(tintervals)
 
     @property
-    def available(self) -> None:
+    def available(self) -> t.Collection[str]:
+        """Generate list of available signals in the first chain."""
         return self.fsignal.available
 
     @property
     def available_grouped(self) -> None:
+        """Display available signals and the number of chains for each."""
         if not hasattr(self, "_available_grouped"):
             self._available_grouped = Counter(
                 [x for s in self.chainers.values() for x in s.available]
             )
-
         for s, n in self._available_grouped.items():
             print(f"{s} - {n}")
 
     @property
     def datasets(self) -> None:
-        """Print available datasets in first Chainer instance."""
+        """Print available data sets in the first chain."""
         return self.fsignal.datasets
 
     @abstractproperty
@@ -83,141 +85,75 @@ class Grouper(ABC):
         standard: t.Optional[bool] = False,
         **kwargs,
     ):
-        """Concate
+        """
+        Concatenate data for one signal from different h5 files, with
+        one h5 file per position, into a dataframe.
 
         Parameters
         ----------
         path : str
-            signal address within h5py file.
-        reduce_cols : bool
-            Whether or not to collapse columns into a single one.
-        axis : int
-            Concatenation axis.
+           Signal location within h5py file
         pool : int
-            Number of threads used. If 0 or None only one core is used.
+           Number of threads used; if 0 or None only one core is used
+        mode: str
+        standard: boolean
         **kwargs : key, value pairings
-            Named arguments to pass to concat_ind_function.
+           Named arguments for concat_ind_function
 
         Examples
         --------
-        FIXME: Add docs.
+        >>> record = grouper.concat_signal("extraction/GFP/max/median")
         """
         if path.startswith("/"):
             path = path.strip("/")
-
-        sitems = self.filter_path(path)
+        good_chains = self.filter_chains(path)
         if standard:
             fn_pos = concat_standard
         else:
             fn_pos = concat_signal_ind
             kwargs["mode"] = mode
-
-        kymographs = self.pool_function(
+        records = self.pool_function(
             path=path,
             f=fn_pos,
             pool=pool,
-            chainers=sitems,
+            chainers=good_chains,
             **kwargs,
         )
-
+        # check for errors
         errors = [
-            k
-            for kymo, k in zip(kymographs, self.chainers.keys())
-            if kymo is None
+            k for kymo, k in zip(records, self.chainers.keys()) if kymo is None
         ]
-        kymographs = [kymo for kymo in kymographs if kymo is not None]
+        records = [record for record in records if record is not None]
         if len(errors):
             print("Warning: Positions contain errors {errors}")
-
-        assert len(kymographs), "All datasets contain errors"
-
-        concat_sorted = (
-            pd.concat(kymographs, axis=0)
-            .reorder_levels(
+        assert len(records), "All data sets contain errors"
+        # combine into one dataframe
+        concat = pd.concat(records, axis=0)
+        if len(concat.index.names) > 4:
+            # reorder levels in the multi-index dataframe when mother_label is present
+            concat = concat.reorder_levels(
                 ("group", "position", "trap", "cell_label", "mother_label")
             )
-            .sort_index()
-        )
+        concat_sorted = concat.sort_index()
         return concat_sorted
 
-    def filter_path(self, path: str) -> t.Dict[str, Chainer]:
-        # Check the path is in a given signal
-        sitems = {
+    def filter_chains(self, path: str) -> t.Dict[str, Chainer]:
+        """Filter chains to those whose data is available in the h5 file."""
+        good_chains = {
             k: v
             for k, v in self.chainers.items()
             if path in [*v.available, *v.common_chains]
         }
-        nchains_dif = len(self.chainers) - len(sitems)
+        nchains_dif = len(self.chainers) - len(good_chains)
         if nchains_dif:
             print(
                 f"Grouper:Warning: {nchains_dif} chains do not contain"
                 f" channel {path}"
             )
-
         assert len(
-            sitems
+            good_chains
         ), f"No valid dataset to use. Valid datasets are {self.available}"
-
-        return sitems
-
-    @property
-    def nmembers(self) -> t.Dict[str, int]:
-        # Return the number of positions belonging to each group
-        return Counter(self.positions_groups.values())
-
-    @property
-    def ntraps(self):
-        for pos, s in self.chainers.items():
-            with h5py.File(s.filename, "r") as f:
-                print(pos, f["/trap_info/trap_locations"].shape[0])
-
-    @property
-    def ntraps_by_pos(self) -> t.Dict[str, int]:
-        # Return total number of traps grouped
-        ntraps = {}
-        for pos, s in self.chainers.items():
-            with h5py.File(s.filename, "r") as f:
-                ntraps[pos] = f["/trap_info/trap_locations"].shape[0]
-
-        ntraps_by_pos = {k: 0 for k in self.groups}
-        for posname, vals in ntraps.items():
-            ntraps_by_pos[self.positions_groups[posname]] += vals
-
-        return ntraps_by_pos
-
-    def traplocs(self):
-        d = {}
-        for pos, s in self.chainers.items():
-            with h5py.File(s.filename, "r") as f:
-                d[pos] = f["/trap_info/trap_locations"][()]
-        return d
-
-    @property
-    def groups(self) -> t.Tuple[str]:
-        # Return groups sorted alphabetically
-        return tuple(sorted(set(self.positions_groups.values())))
-
-    @property
-    def positions(self) -> t.Tuple[str]:
-        # Return positions sorted alphabetically
-        return tuple(sorted(set(self.positions_groups.keys())))
-
-    def ncells(
-        self, path="extraction/general/None/area", mode="retained", **kwargs
-    ) -> t.Dict[str, int]:
-        """
-        Returns number of cells retained per position in base channel
-        """
-        return (
-            self.concat_signal(path=path, mode=mode, **kwargs)
-            .groupby("group")
-            .apply(len)
-            .to_dict()
-        )
-
-    @property
-    def nretained(self) -> t.Dict[str, int]:
-        return self.ncells()
+        return good_chains
 
     def pool_function(
         self,
@@ -227,18 +163,13 @@ class Grouper(ABC):
         chainers: t.Dict[str, Chainer] = None,
         **kwargs,
     ):
-        """
-        Wrapper to add support for threading to process independent chains.
-        Particularly useful when aggregating multiple elements.
-        """
+        """Enable different threads for independent chains, particularly useful when aggregating multiple elements."""
         if pool is None:
-            pool = 8
+            pass
         chainers = chainers or self.chainers
-
         if pool:
-
             with Pool(pool) as p:
-                kymographs = p.map(
+                records = p.map(
                     lambda x: f(
                         path=path,
                         chainer=x[1],
@@ -249,7 +180,7 @@ class Grouper(ABC):
                     chainers.items(),
                 )
         else:
-            kymographs = [
+            records = [
                 f(
                     path=path,
                     chainer=chainer,
@@ -259,11 +190,73 @@ class Grouper(ABC):
                 )
                 for name, chainer in self.chainers.items()
             ]
+        return records
 
-        return kymographs
+    @property
+    def nmembers(self) -> t.Dict[str, int]:
+        """Get the number of positions belonging to each group."""
+        return Counter(self.positions_groups.values())
+
+    @property
+    def ntiles(self):
+        """Get total number of tiles per position (h5 file)."""
+        for pos, s in self.chainers.items():
+            with h5py.File(s.filename, "r") as f:
+                print(pos, f["/trap_info/trap_locations"].shape[0])
+
+    @property
+    def ntiles_by_group(self) -> t.Dict[str, int]:
+        """Get total number of tiles per group."""
+        ntiles = {}
+        for pos, s in self.chainers.items():
+            with h5py.File(s.filename, "r") as f:
+                ntiles[pos] = f["/trap_info/trap_locations"].shape[0]
+        ntiles_by_group = {k: 0 for k in self.groups}
+        for posname, vals in ntiles.items():
+            ntiles_by_group[self.positions_groups[posname]] += vals
+        return ntiles_by_group
+
+    @property
+    def tilelocs(self) -> t.Dict[str, np.ndarray]:
+        """Get the locations of the tiles for each position as a dictionary."""
+        d = {}
+        for pos, s in self.chainers.items():
+            with h5py.File(s.filename, "r") as f:
+                d[pos] = f["/trap_info/trap_locations"][()]
+        return d
+
+    @property
+    def groups(self) -> t.Tuple[str]:
+        """Get groups, sorted alphabetically."""
+        return tuple(sorted(set(self.positions_groups.values())))
+
+    @property
+    def positions(self) -> t.Tuple[str]:
+        """Get positions, sorted alphabetically."""
+        return tuple(sorted(set(self.positions_groups.keys())))
+
+    def ncells(
+        self,
+        path="extraction/general/None/area",
+        mode="retained",
+        **kwargs,
+    ) -> t.Dict[str, int]:
+        """Get number of cells retained per position in base channel as a dictionary."""
+        return (
+            self.concat_signal(path=path, mode=mode, **kwargs)
+            .groupby("group")
+            .apply(len)
+            .to_dict()
+        )
+
+    @property
+    def nretained(self) -> t.Dict[str, int]:
+        """Get number of cells retained per position in base channel as a dictionary."""
+        return self.ncells()
 
     @property
     def channels(self):
+        """Get unique channels for all chains as a set."""
         return set(
             [
                 channel
@@ -274,19 +267,23 @@ class Grouper(ABC):
 
     @property
     def stages_span(self):
+        # FAILS on my example
         return self.fsignal.stages_span
 
     @property
     def max_span(self):
+        # FAILS on my example
         return self.fsignal.max_span
 
     @property
-    def tinterval(self):
-        return self.fsignal.tinterval
+    def stages(self):
+        # FAILS on my example
+        return self.fsignal.stages
 
     @property
-    def stages(self):
-        return self.fsignal.stages
+    def tinterval(self):
+        """Get interval between time points."""
+        return self.fsignal.tinterval
 
 
 class MetaGrouper(Grouper):
@@ -296,51 +293,49 @@ class MetaGrouper(Grouper):
 
 
 class NameGrouper(Grouper):
-    """Group a set of positions using a subsection of the name."""
+    """Group a set of positions with a shorter version of the group's name."""
 
-    def __init__(self, dir, criteria=None):
+    def __init__(self, dir, name_inds=(0, -4)):
+        """Define the indices to slice names."""
         super().__init__(dir=dir)
-
-        if criteria is None:
-            criteria = (0, -4)
-        self.criteria = criteria
+        self.name_inds = name_inds
 
     @property
     def positions_groups(self) -> t.Dict[str, str]:
+        """Get a dictionary with the positions as keys and groups as items."""
         if not hasattr(self, "_positions_groups"):
             self._positions_groups = {}
             for name in self.chainers.keys():
                 self._positions_groups[name] = name[
-                    self.criteria[0] : self.criteria[1]
+                    self.name_inds[0] : self.name_inds[1]
                 ]
-
         return self._positions_groups
 
 
 class phGrouper(NameGrouper):
-    """Grouper for pH calibration experiments where all surveyed media pH
-    values are within a single experiment."""
+    """Grouper for pH calibration experiments where all surveyed media pH values are within a single experiment."""
 
-    def __init__(self, dir, criteria=(3, 7)):
-        super().__init__(dir=dir, criteria=criteria)
+    def __init__(self, dir, name_inds=(3, 7)):
+        """Initialise via NameGrouper."""
+        super().__init__(dir=dir, name_inds=name_inds)
 
     def get_ph(self) -> None:
+        """Find the pH from the group names and store as a dictionary."""
         self.ph = {gn: self.ph_from_group(gn) for gn in self.positions_groups}
 
     @staticmethod
-    def ph_from_group(group_name: int) -> float:
-        if group_name.startswith("ph_"):
+    def ph_from_group(group_name: str) -> float:
+        """Find the pH from the name of a group."""
+        if group_name.startswith("ph_") or group_name.startswith("pH_"):
             group_name = group_name[3:]
-
         return float(group_name.replace("_", "."))
 
-    def aggregate_multichains(self, paths: list) -> pd.DataFrame:
-        """Accumulate multiple chains."""
-
+    def aggregate_multichains(self, signals: list) -> pd.DataFrame:
+        """Get data from a list of signals and combine into one multi-index dataframe with 'media-pH' included."""
         aggregated = pd.concat(
             [
-                self.concat_signal(path, reduce_cols=np.nanmean)
-                for path in paths
+                self.concat_signal(signal, reduce_cols=np.nanmean)
+                for signal in signals
             ],
             axis=1,
         )
@@ -355,7 +350,6 @@ class phGrouper(NameGrouper):
             name="media_pH",
         )
         aggregated = pd.concat((aggregated, ph), axis=1)
-
         return aggregated
 
 
@@ -366,13 +360,13 @@ def concat_standard(
     position: t.Optional[str] = None,
     **kwargs,
 ) -> pd.DataFrame:
-
     combined = chainer.get(path, **kwargs).copy()
     combined["position"] = position
     combined["group"] = group
     combined.set_index(["group", "position"], inplace=True, append=True)
-    combined.index = combined.index.copy().swaplevel(-2, 0).swaplevel(-1, 1)
-
+    combined.index = combined.index.reorder_levels(
+        ("group", "position", "trap", "cell_label", "mother_label")
+    )
     return combined
 
 
@@ -384,23 +378,34 @@ def concat_signal_ind(
     position=None,
     **kwargs,
 ) -> pd.DataFrame:
-    """Core function that handles retrieval of an individual signal, applies
-    filtering if requested and adjusts indices."""
+    """
+    Retrieve an individual signal.
+
+    Applies filtering if requested and adjusts indices.
+    """
     if position is None:
+        # name of h5 file
         position = chainer.stem
     if mode == "retained":
         combined = chainer.retained(path, **kwargs)
-    if mode == "mothers":
-        raise (NotImplementedError)
     elif mode == "raw":
         combined = chainer.get_raw(path, **kwargs)
+    elif mode == "daughters":
+        combined = chainer.get_raw(path, **kwargs)
+        combined = combined.loc[
+            combined.index.get_level_values("mother_label") > 0
+        ]
     elif mode == "families":
         combined = chainer[path]
-    combined["position"] = position
-    combined["group"] = group
-    combined.set_index(["group", "position"], inplace=True, append=True)
-    combined.index = combined.index.swaplevel(-2, 0).swaplevel(-1, 1)
-
+    else:
+        raise Exception(f"{mode} not recognised.")
+    if combined is not None:
+        # adjust indices
+        combined["position"] = position
+        combined["group"] = group
+        combined.set_index(["group", "position"], inplace=True, append=True)
+        combined.index = combined.index.swaplevel(-2, 0).swaplevel(-1, 1)
+    # should there be an error message if None is returned?
     return combined
 
 
@@ -409,6 +414,14 @@ class MultiGrouper:
     folder."""
 
     def __init__(self, source: Union[str, list]):
+        """
+        Create NameGroupers for each experiment.
+
+        Parameters
+        ----------
+        source: list of str
+            List of folders, one per experiment, containing h5 files.
+        """
         if isinstance(source, str):
             source = Path(source)
             self.exp_dirs = list(source.glob("*"))
@@ -420,43 +433,38 @@ class MultiGrouper:
 
     @property
     def available(self) -> None:
+        """Print available signals and number of chains, one per position, for each Grouper."""
         for gpr in self.groupers:
             print(gpr.available_grouped)
 
     @property
     def sigtable(self) -> pd.DataFrame:
-        """Generate a matrix containing the number of datasets for each signal
-        and experiment."""
+        """Generate a table showing the number of positions, or h5 files, available for each signal with one column per experiment."""
 
         def regex_cleanup(x):
             x = re.sub(r"extraction\/", "", x)
             x = re.sub(r"postprocessing\/", "", x)
             x = re.sub(r"\/max", "", x)
-
             return x
 
         if not hasattr(self, "_sigtable"):
             raw_mat = [
-                [s.available for s in gpr.chains.values()]
+                [s.available for s in gpr.chainers.values()]
                 for gpr in self.groupers
             ]
             available_grouped = [
                 Counter([x for y in grp for x in y]) for grp in raw_mat
             ]
-
             nexps = len(available_grouped)
             sigs_idx = list(
                 set([y for x in available_grouped for y in x.keys()])
             )
             sigs_idx = [regex_cleanup(x) for x in sigs_idx]
-
             nsigs = len(sigs_idx)
-
             sig_matrix = np.zeros((nsigs, nexps))
             for i, c in enumerate(available_grouped):
                 for k, v in c.items():
                     sig_matrix[sigs_idx.index(regex_cleanup(k)), i] = v
-
             sig_matrix[sig_matrix == 0] = np.nan
             self._sigtable = pd.DataFrame(
                 sig_matrix,
@@ -465,8 +473,9 @@ class MultiGrouper:
             )
         return self._sigtable
 
-    def sigtable_plot(self) -> None:
-        """Plot number of chains for all available experiments.
+    def _sigtable_plot(self) -> None:
+        """
+        Plot number of chains for all available experiments.
 
         Examples
         --------
@@ -486,27 +495,30 @@ class MultiGrouper:
         path: Union[str, list],
         **kwargs,
     ) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
-        """Aggregate chains from multiple Groupers (and thus experiments)
+        """
+        Aggregate chains, one per position, from multiple Groupers, one per experiment.
 
         Parameters
         ----------
-        chains : Union[str, list]
-            string or list of strings indicating the signal(s) to fetch.
-        **kwargs : keyword arguments to pass to Grouper.concat_signal
-            Customise the filters and format to fetch chains.
+        path : Union[str, list]
+            String or list of strings indicating the signal(s) to fetch.
+        **kwargs :
+            Passed to Grouper.concat_signal.
 
         Returns
         -------
-        Union[pd.DataFrame, Dict[str, pd.DataFrame]]
-            DataFrame or list of DataFrames
+        concatenated: Union[pd.DataFrame, Dict[str, pd.DataFrame]]
+            A multi-index dataFrame or a dictionary of multi-index dataframes, one per signal
 
         Examples
         --------
-        FIXME: Add docs.
+        >>> mg = MultiGrouper(["pHCalibrate7_24", "pHCalibrate6_7"])
+        >>> p405 = mg.aggregate_signal("extraction/pHluorin405_0_4/max/median")
+        >>> p588 = mg.aggregate_signal("extraction/pHluorin488_0_4/max/median")
+        >>> ratio = p405 / p488
         """
         if isinstance(path, str):
             path = [path]
-
         sigs = {s: [] for s in path}
         for s in path:
             for grp in self.groupers:
@@ -516,18 +528,14 @@ class MultiGrouper:
                         [(grp.name, *x) for x in sigset.index],
                         names=("experiment", *sigset.index.names),
                     )
-
                     sigset.index = new_idx
                     sigs[s].append(sigset)
                 except Exception as e:
                     print("Grouper {} failed: {}".format(grp.name, e))
-                    # raise (e)
-
-        concated = {
+        concatenated = {
             name: pd.concat(multiexp_sig)
             for name, multiexp_sig in sigs.items()
         }
-        if len(concated) == 1:
-            concated = list(concated.values())[0]
-
-        return concated
+        if len(concatenated) == 1:
+            concatenated = list(concatenated.values())[0]
+        return concatenated
